
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, CHAT_FRAMES, ChatHistory_GetAccessID, ChatTypeInfo, COMBATLOG, select, SetItemRef, UNKNOWN

local ChatHistory_GetChatType = ChatHistory_GetChatType
local format = format
local string = string
local table = table
local tonumber = tonumber
local type = type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat History")
local Social = LibStub("SixSocial-1.0")

local MAX_LINES = 128
local history = {}

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function GetBattleTagLink(accountID)
	local battleTag = (Social:GetFriendByAccountID(tonumber(accountID)) or {}).battleTag
	return format("|HbattleTag:%s|h[%s]|h", battleTag or "", string.split("#", battleTag or UNKNOWN))
end

local function HandleBattleTag(battleTag, button)
	local friend = Social:GetFriendByBattleTag(battleTag)
	if (not friend) then return end
	SetItemRef("BNplayer:" .. friend.accountName .. ":" .. friend.accountID, friend.accountName, button)
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self.db = D.db:RegisterNamespace("Chat History", {})
	history = self.db.global

	if not A:IsModuleDisabled(self:GetName()) then
		self:ConfigureChatFrames()
	end
end

function M:OnEnable()
	self:ConfigureChatFrames()
	self:RestoreChatFrames()

	if not self:IsHooked("SetItemRef") then
		self:SecureHook("SetItemRef")
	end
end

function M:OnDisable()
	self:ForgetChatFrames()
end

function M:ConfigureChatFrames()
	for index = 1, #CHAT_FRAMES do
		local frameName = "ChatFrame" .. index
		self:ConfigureChatFrame(_G[frameName], frameName, history[frameName])
	end
end

function M:ConfigureChatFrame(chatFrame, frameName, messages)
	if not chatFrame or IsCombatLog(chatFrame) or IsVoiceTranscription(chatFrame) then
		return
	end

	if not messages then
		history[frameName] = {}
	end

	if not self:IsHooked(chatFrame, "AddMessage") then
		self:SecureHook(chatFrame, "AddMessage")
	end
end

function M:RestoreChatFrames()
	for index = 1, #CHAT_FRAMES do
		local frameName = "ChatFrame" .. index
		self:RestoreChatFrame(_G[frameName], frameName, history[frameName])
	end
end

function M:RestoreChatFrame(chatFrame, frameName, messages)
	if not self:IsHooked(chatFrame, "AddMessage") or not messages or #messages == 0 then
		return
	end

	chatFrame:Clear()

	for index = #messages, 1, -1 do
		local message = messages[index]
		local accessID = message.type and ChatHistory_GetAccessID(message.type, "", "")
		local info = message.type and ChatTypeInfo[message.type] or {r = message.r, g = message.g, b = message.b}

		chatFrame:BackFillMessage(message.text, info.r, info.g, info.b, accessID, message.backFill, accessID, message.extraData)
	end
end

function M:ForgetChatFrames()
	for index = 1, #CHAT_FRAMES do
		history["ChatFrame" .. index] = nil
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:AddMessage(frame, text, r, g, b, typeID, backFill, accessID, extraData)
	local messages = history[frame:GetName()]

	if type(text) ~= "string" then
		return
	end

	table.insert(messages, {
		text = text:gsub("|HBNplayer:[^:]-:(%d+):.-|h.-|h", GetBattleTagLink),
		type = ChatHistory_GetChatType(accessID) or C.CHAT_TYPE_ID_TO_KEY[typeID] or "",
		r = r,
		g = g,
		b = b,
		backFill = backFill,
		extraData = extraData,
	})

	if #messages > MAX_LINES then
		table.remove(messages, 1)
	end
end

function M:SetItemRef(link, _, button)
	local type, value = string.split(":", link, 2)

	if type == "battleTag" then
		HandleBattleTag(value, button)
	end
end
